
#include <lubishev/DBDRecords.h>
#include <lubishev/DBSQLite.h>
#include <lubishev/MainWindow.h>
#include <lubishev/Settings.h>

#include <ctime>
#include <QCloseEvent>
#include <QDesktopServices>
#include <QMessageBox>
#include <QUrl>

#include <iostream>

namespace lubishev {

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent) {
    // Load UI header file.
    setupUi(this);
    mDB = new DBDRecords;

    settingsLoad();
    // File.
    connect(mActionQuit, SIGNAL(triggered()), this, SLOT(close()));
    // Tools.
    connect(mActionSettings, SIGNAL(triggered()), this, SLOT(settings()));
    // Help.
    connect(mActionWebHome, SIGNAL(triggered()), this, SLOT(webHome()));
    connect(mActionAbout,   SIGNAL(triggered()), this, SLOT(about()));
    connect(mActionAboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    mScene = new Scene(mDB);
    setCentralWidget(mScene);
    mScene->setIntervalNow();
    mBtnCursorTime = new QPushButton(mStatusBar);
    mStatusBar->insertWidget(0, mBtnCursorTime);
    // Other.
    connect(mScene, SIGNAL(cursorTimeChanged(time_t)),
            this,   SLOT(onCursorTimeChange(time_t)));
    connect(mBtnCursorTime, SIGNAL(clicked()), this, SLOT(onBtnCursor()));
    mBtnCursorTime->setFocusPolicy(Qt::ClickFocus);
}

MainWindow::~MainWindow() {
    settingsSave();
    delete mScene;
    delete mDB;
}

void MainWindow::settingsLoad() {
    bool max = mSettings.value("wnd.max", false).toBool();
    if (max)
        setWindowState(windowState() | Qt::WindowMaximized);
    else
        setGeometry(mSettings.value("wnd.geometry", QRect(0, 0, 640, 480)).toRect());
    mDB->setDir(mSettings.value("db.dir", ".").toString());
}

void MainWindow::settingsSave() {
    mSettings.setValue("wnd.max", windowState() && Qt::WindowMaximized);
    mSettings.setValue("wnd.geometry", geometry());
}

void MainWindow::about() {
    QString msg = tr(
        "Lubishev is a logger application to track / plan your "
        "life time. Inspired by Lubishev A. A. who has been tracking "
        "his life time for 56 years as precisely as up to 5 "
        "minute interval.\n\n"
        "Licensed under GPL v3.\n"
        "(c) 2010 - 2013, Michael Kapelko (kornerr AT Google mail)\n\n"
        "Build: %1 %2 GCC-%3 %4").arg(__DATE__)
                                 .arg(__TIME__)
                                 .arg(__VERSION__)
                                 .arg(
#ifdef WIN32
                                      "Windows32"
#elif defined(_LP64)
                                      "Linux64"
#else
                                      "Linux32"
#endif
                                      );
    QMessageBox::about(this, tr("Lubishev"), msg);
}

void MainWindow::onBtnCursor() {
    bool state = mScene->calendarIsVisible();
    mScene->setCalendarVisible(!state);
}

void MainWindow::onCursorTimeChange(time_t cursor) {
    struct tm * t = localtime(&cursor);
    mBtnCursorTime->setText(QString("%1-%2-%3").arg(t->tm_year + 1900)
                                               .arg(t->tm_mon + 1)
                                               .arg(t->tm_mday));
}

void MainWindow::settings() {
    Settings dlg(this, mSettings);
    dlg.exec();
}

void MainWindow::webHome() {
    QDesktopServices::openUrl(QUrl("http://bitbucket.org/kornerr/lubishev"));
}

} // namespace lubishev

